package precision;

import precision.model.IPrecisionConfiguration;

public final class PrecisionConfiguration implements IPrecisionConfiguration
{
	private final int accuracy;
	
	private final double distanceDelta;
	
	private final long iEpsilon;
	
	private final double epsilon;

	public PrecisionConfiguration(int accuracy, double distanceDelta)
	{
		if (accuracy < 1 || accuracy > 15)
			throw new IllegalArgumentException("The number of trusted decimal digits doesn't fit the range [1; 15].");
		
		this.accuracy = accuracy;
		this.iEpsilon = (long)Math.pow(10, accuracy);
		this.epsilon = 1.0 / this.iEpsilon;
		
		this.distanceDelta = distanceDelta;
	}
	
	public PrecisionConfiguration()
	{
		this(4, 0.00009);
	}
	
	@Override
	public int accuracy()
	{
		return this.accuracy;
	}
	
	@Override
	public long iEpsilon()
	{
		return this.iEpsilon;
	}
	
	@Override
	public double epsilon() 
	{
		// TODO Auto-generated method stub
		return 0;
	}
	
	@Override
	public double distanceDelta() 
	{
		return this.distanceDelta;
	}

	@Override
	public int compareDistance(double distance1, double distance2)
	{
		if (Math.abs(distance1 - distance2) < this.distanceDelta)
			return 0;
		
		if (distance1 > distance2)
			return 1;
		else
			return -1;
	}
	
	@Override
	public int compareWithAccuracy(double value1, double value2)
	{
		if (Math.abs(value1 - value2) < this.epsilon)
			return 0;
		
		if (value1 > value2)
			return 1;
		else
			return -1;
	}

}
