package spatial;

import java.security.InvalidParameterException;

import spatial.geometry.Vector3d;
import spatial.model.IGeometry;
import spatial.model.elements.INode;
import utils.StringUtils;

final class Node implements INode 
{
	
	private final Vector3d vector;
	
	private final String name;
	
	public Node(String name, Vector3d vector)
	{
		if (StringUtils.isNullOrEmpty(name))
			throw new InvalidParameterException("name");
		if (vector == null)
			throw new InvalidParameterException("vector");
		
		this.name = name;
		this.vector = vector;
	}
	
	@Override
	public final String name() 
	{
		return this.name;
	}
	
	@Override
	public final IGeometry geometry() 
	{
		return this.vector;
	}
	
	@Override
	public final int hashCode() 
	{
		return (41 * 43 + this.vector.hashCode()) * 43 + this.name.hashCode();
	}
	
	@Override
	public final boolean equals(Object other) 
	{	
		if (other == null || this.getClass() != other.getClass())
			return false;
		if (this == other)
			return true;
		Node otherNode = (Node) other;

		return this.vector.equals(otherNode.vector) && this.name.equals(otherNode.name);
	}

	@Override
	public final String toString()
	{
		return this.name + "(" + this.vector.toString() + ")";
	}

}
