package spatial.algorithms;

import java.util.ArrayList;

import precision.model.IPrecisionConfiguration;

public abstract class AlgorithmBase<TResult>
{
	protected final IPrecisionConfiguration precisionConfiguration;
	
	private final ArrayList<String> errors;
	
	protected AlgorithmBase(IPrecisionConfiguration precisionConfiguration)
	{
		if (precisionConfiguration == null)
			throw new IllegalArgumentException("PrecisionConfiguration is not defined.");
		
		this.precisionConfiguration = precisionConfiguration;
		
		this.errors = new ArrayList<String>();
	}
	
	protected boolean hasErrors()
	{
		return !this.errors.isEmpty();
	}
	
	protected String[] errors()
	{
		return (String[]) this.errors.toArray();
	}
	
	protected final TResult calculate()
	{
		try
		{
			this.calculateInternal();
			
			if (!this.hasErrors())
			{
				return this.constructResult();
			} 
			else
			{
				return this.constructErrorsResult();
			}
		} 
		catch (Exception exception)
		{
			this.errors.add(exception.getMessage());
			return this.constructErrorsResult();
		}
	}
	
	protected abstract void calculateInternal();

	protected abstract TResult constructResult();
	
	protected abstract TResult constructErrorsResult();
	
	protected void addErrors(String[] errors)
	{
		for (int i = 0; i < errors.length; i++)
		{
			this.errors.add(errors[i]);
		}
	}
	
	protected void addError(String error)
	{
		this.errors.add(error);
	}

}
