package spatial.algorithms.csTransform;

import org.ejml.data.DMatrix;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.geometry.CoordinateSystem3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.csTransformation.ICsTransformationAlgorithm;
import spatial.model.algorithms.csTransformation.ICsTransformationResult;

public class CsTransformationAlgorithm extends AlgorithmBase<ICsTransformationResult> implements ICsTransformationAlgorithm
{
	//start: algorithm input arguments
	
	private Vector3d normalPlaneOrt;
	
	private Vector3d inPlaneOrt;
	
	private Vector3d localCSOrigin;
	
	private Vector3d[] globalPoints;

	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private DMatrix rotationMatrix;
	
	private Vector3d[] localPoints;
	
	//end: algorithm result arguments
	
	public CsTransformationAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public ICsTransformationResult calculate(Vector3d normalPlaneOrt, Vector3d inPlaneOrt, Vector3d localCSOrigin, Vector3d[] globalCsPoints)
	{
		if (normalPlaneOrt == null)
			throw new IllegalArgumentException("The Normal Plane Coordinate Ort is not defined.");
		if (inPlaneOrt == null)
			throw new IllegalArgumentException("The In Plane Coordinate Ort is not defined.");
		if (localCSOrigin == null)
			throw new IllegalArgumentException("The Local CS Origin Point is not defined.");
		if (globalCsPoints == null)
			throw new IllegalArgumentException("The Global Points are not defined.");

		this.normalPlaneOrt = normalPlaneOrt;
		this.inPlaneOrt = inPlaneOrt;
		this.localCSOrigin = localCSOrigin;
		this.globalPoints = globalCsPoints;
		
		return super.calculate();
	}
	
	@Override
	protected void calculateInternal()
	{
		CoordinateSystem3d cs = CoordinateSystem3d.create(this.localCSOrigin, this.inPlaneOrt, this.normalPlaneOrt, this.precisionConfiguration);
		
		this.localPoints = new Vector3d[this.globalPoints.length];
		
		for (int i = 0; i < this.globalPoints.length; i++)
		{
			this.localPoints[i] = cs.toLocal(this.globalPoints[i], this.precisionConfiguration);
		}
		
		this.rotationMatrix = cs.coordinateSystemBasisMatrix();
	}

	@Override
	protected ICsTransformationResult constructResult()
	{
		return new CsTransformationResult(this.rotationMatrix, this.localPoints);
	}

	@Override
	protected ICsTransformationResult constructErrorsResult()
	{
		return new CsTransformationResult(this.errors());
	}
}
