package spatial.algorithms.csTransform;

import org.ejml.data.DMatrix;

import spatial.algorithms.AlgorithmResultBase;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.csTransformation.ICsTransformationResult;

final class CsTransformationResult extends AlgorithmResultBase implements ICsTransformationResult
{

	private final DMatrix rotationMatrix;
	
	private final Vector3d[] localPoints;
	
	public CsTransformationResult(DMatrix rotationMatrix, Vector3d[] localPoints)
	{
		if (rotationMatrix == null)
			throw new IllegalArgumentException("The Rotation Matrix is not defined.");
		if (localPoints == null)
			throw new IllegalArgumentException("The Local points are not defined.");

		this.rotationMatrix = rotationMatrix;
		this.localPoints = localPoints;
	}

	public CsTransformationResult(String[] errors)
	{
		super(errors);
		
		this.rotationMatrix = null;
		this.localPoints = null;
	}

	@Override
	public DMatrix rotationMatrix()
	{
		return this.rotationMatrix;
	}

	@Override
	public Vector3d[] localPoints()
	{
		return this.localPoints;
	}

}
