package spatial.algorithms.edgeToEdgeLocation;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmResultBase;
import spatial.geometry.Vector2d;
import spatial.model.algorithms.edgeToEdgeLocation.EdgeToEdgeLocationKind;
import spatial.model.algorithms.edgeToEdgeLocation.IEdgeToEdgeLocationResult;

final class EdgeToEdgeLocationResult extends AlgorithmResultBase implements IEdgeToEdgeLocationResult
{
	private final EdgeToEdgeLocationKind locationCode;
	
	private final double r0;
	
	private final double t0;
	
	private final double rc;
	
	private final double rd;
	
	private final Vector2d intersectionPoint;
	
	private final IPrecisionConfiguration precisionConfiguration;
	
	public EdgeToEdgeLocationResult(EdgeToEdgeLocationKind locationCode, double r0, double t0, double rc, double rd, Vector2d intersectionPoint, IPrecisionConfiguration precisionConfiguration)
	{
		this.locationCode = locationCode;
		this.r0 = r0;
		this.t0 = t0;
		this.rc = rc;
		this.rd = rd;
		this.intersectionPoint = intersectionPoint;
		this.precisionConfiguration = precisionConfiguration;
	}

	public EdgeToEdgeLocationResult(String[] errors)
	{
		super(errors);
		
		this.locationCode = EdgeToEdgeLocationKind.NotSet;
		this.r0 = 0;
		this.t0 = 0;
		this.rc = 0;
		this.rd = 0;
		this.intersectionPoint = null;
		this.precisionConfiguration = null;
	}
	
	@Override
	public final EdgeToEdgeLocationKind locationCode()
	{
		return this.locationCode;
	}

	@Override
	public final Vector2d intersectionPoint()
	{
		return this.intersectionPoint;
	}

	@Override
	public final double r0()
	{
		return this.r0;
	}

	@Override
	public final double t0()
	{
		return this.t0;
	}

	@Override
	public final double rc()
	{
		return this.rc;
	}

	@Override
	public final double rd()
	{
		return this.rd;
	}

	@Override
	public final boolean edgesIntersectInside()
	{
		if (this.locationCode != EdgeToEdgeLocationKind.NotParallel)
			return false;
		
		if (this.precisionConfiguration.compareWithAccuracy(this.r0, 0) == 1 && this.precisionConfiguration.compareWithAccuracy(this.r0, 1) == -1)
		{
			if (this.precisionConfiguration.compareWithAccuracy(this.t0, 0) == 1 && this.precisionConfiguration.compareWithAccuracy(this.t0, 1) == -1 ||
					this.precisionConfiguration.compareWithAccuracy(this.t0, 0) == 0 || this.precisionConfiguration.compareWithAccuracy(this.t0, 1) == 0)
			{
				return true;
			} 
			else
			{
				return false;
			}
		} 
		else if (this.precisionConfiguration.compareWithAccuracy(this.t0, 0) == 1 && this.precisionConfiguration.compareWithAccuracy(this.t0, 1) == -1)
		{
			if (this.precisionConfiguration.compareWithAccuracy(this.r0, 0) == 1 && this.precisionConfiguration.compareWithAccuracy(this.r0, 1) == -1 ||
					this.precisionConfiguration.compareWithAccuracy(this.r0, 0) == 0 || this.precisionConfiguration.compareWithAccuracy(this.r0, 1) == 0)
			{
				return true;
			} 
			else
			{
				return false;
			}
		} else
			return false;
	}

	@Override
	public final boolean edgesIntersectAtNode()
	{
		if (this.locationCode == EdgeToEdgeLocationKind.OverlapAtNode)
		{
			return true;
		} 
		else if (this.locationCode == EdgeToEdgeLocationKind.NotParallel)
		{
			if ((this.precisionConfiguration.compareWithAccuracy(this.r0, 0) == 0 || this.precisionConfiguration.compareWithAccuracy(this.r0, 1) == 0) &&
					(this.precisionConfiguration.compareWithAccuracy(this.t0, 0) == 0 || this.precisionConfiguration.compareWithAccuracy(this.t0, 1) == 0))
				return true;
			else
				return false;
		}
		else
		{
			return false;
		}
	}
	
}