package spatial.algorithms.pointToEdgeDistance;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.geometry.Edge3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToEdgeDistance.IPointToEdgeDistanceAlgorithm;
import spatial.model.algorithms.pointToEdgeDistance.IPointToEdgeDistanceResult;
import spatial.model.algorithms.pointToEdgeDistance.PointToEdgeLocationKind;

public final class PointToEdgeDistanceAlgorithm extends AlgorithmBase<IPointToEdgeDistanceResult> implements IPointToEdgeDistanceAlgorithm
{
	//start: algorithm input arguments
	
	private Vector3d point;
	
	private Edge3d edge;
	
	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private PointToEdgeLocationKind distancePointLocationKind;
	
	private Vector3d edgeDistancePoint;
	
	private Vector3d projectionPoint;
	
	private double distance;
	
	//end: algorithm result arguments
	
	public PointToEdgeDistanceAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}
	
	@Override
	public IPointToEdgeDistanceResult calculate(Edge3d edge, Vector3d testPoint)
	{
		if (edge == null)
			throw new IllegalArgumentException("The Edge is not defined.");
		if (testPoint == null)
			throw new IllegalArgumentException("The Test Point is not defined.");
		
		this.point = testPoint;
		this.edge = edge;
		
		return super.calculate();
	}
	
	@Override
	protected final	IPointToEdgeDistanceResult constructResult()
	{
		return new PointToEdgeDistanceResult(this.distancePointLocationKind, this.edgeDistancePoint, this.projectionPoint, this.distance);
	}
	
	@Override
	protected final IPointToEdgeDistanceResult constructErrorsResult()
	{
		return new PointToEdgeDistanceResult(this.errors());
	}
	
	@Override
	protected final	void calculateInternal()
	{
		Vector3d abVector = this.edge.vectorB().substract(this.edge.vectorA(), this.precisionConfiguration);
		Vector3d atVector = this.point.substract(this.edge.vectorA(), this.precisionConfiguration);
		
		double d1 = abVector.dotProduct(atVector);
		double d2 = abVector.dotProduct(abVector);
		double m = d1 / d2;
		
		Vector3d mabVector = abVector.mul(m, this.precisionConfiguration);
		this.projectionPoint = this.edge.vectorA().add(mabVector, this.precisionConfiguration);
		
		Vector3d t;
		if (m > 0 && m < 1 || this.precisionConfiguration.compareWithAccuracy(0, m) == 0 || this.precisionConfiguration.compareWithAccuracy(1, m) == 0)
		{
			t = atVector.substract(mabVector, this.precisionConfiguration);
			this.edgeDistancePoint = this.projectionPoint;
			this.distancePointLocationKind = PointToEdgeLocationKind.InnerPoint;
		}
		else if (m < 0)
		{
			t = edge.vectorA().substract(point, this.precisionConfiguration);
			this.edgeDistancePoint = edge.vectorA();
			this.distancePointLocationKind = PointToEdgeLocationKind.StartPoint;
		}
		else
		{
			t = edge.vectorB().substract(point, this.precisionConfiguration);
			this.edgeDistancePoint = edge.vectorB();
			this.distancePointLocationKind = PointToEdgeLocationKind.EndPoint;
		}		
		this.distance = Math.sqrt(t.dotProduct(t));
	}
}
