package spatial.algorithms.pointToEdgeDistance;

import spatial.algorithms.AlgorithmResultBase;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToEdgeDistance.IPointToEdgeDistanceResult;
import spatial.model.algorithms.pointToEdgeDistance.PointToEdgeLocationKind;

/**
 * The calculation result of the algorithm "Distance Between a Point and an Edge".
 */
final class PointToEdgeDistanceResult extends AlgorithmResultBase implements IPointToEdgeDistanceResult
{
	private final PointToEdgeLocationKind distancePointLocationKind;
	
	private final Vector3d edgeDistancePoint;
	
	private final Vector3d projectionPoint;
	
	private final double distance;
	
	PointToEdgeDistanceResult(PointToEdgeLocationKind distancePointLocationKind, Vector3d edgeDistancePoint, Vector3d projectionPoint, double distance)
	{
		if (distancePointLocationKind == null)
			throw new IllegalArgumentException("The Distance Kind is not defined.");
		if (edgeDistancePoint == null)
			throw new IllegalArgumentException("The Edge Distance Point is not defined.");
		if (projectionPoint == null)
			throw new IllegalArgumentException("The Projection Point is not defined.");
		
		this.distancePointLocationKind = distancePointLocationKind;
		this.edgeDistancePoint = edgeDistancePoint;
		this.projectionPoint = projectionPoint;
		this.distance = distance;
	}
	
	PointToEdgeDistanceResult(String[] errors)
	{
		super(errors);
		
		this.distancePointLocationKind = PointToEdgeLocationKind.NotSet;
		this.edgeDistancePoint = null;
		this.projectionPoint = null;
		this.distance = 0;
	}
	
	@Override
	public final PointToEdgeLocationKind distancePointLocationKind()
	{
		return this.distancePointLocationKind;
	}
	
	@Override
	public final Vector3d edgeDistancePoint()
	{
		return this.edgeDistancePoint;
	}
	
	@Override
	public final double distance()
	{
		return this.distance;
	}

	@Override
	public Vector3d projectionPoint()
	{
		return this.projectionPoint;
	}
}
