package spatial.algorithms.pointToFaceLocation;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.algorithms.pointToPolygonLocation.PointToPolygonLocationAlgorithm;
import spatial.geometry.Face3d;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToFaceLocation.IPointToFaceLocationAlgorithm;
import spatial.model.algorithms.pointToFaceLocation.IPointToFaceLocationResult;
import spatial.model.algorithms.pointToFaceLocation.PointToFaceLocationKind;
import spatial.model.algorithms.pointToPolygonLocation.IPointToPolygonLocationAlgorithm;
import spatial.model.algorithms.pointToPolygonLocation.IPointToPolygonLocationResult;
import spatial.model.algorithms.pointToPolygonLocation.PointToPolygonLocationKind;

public final class PointToFaceLocationAlgorithm extends AlgorithmBase<IPointToFaceLocationResult> implements IPointToFaceLocationAlgorithm
{
	//start: algorithm input arguments
	
	private Vector3d planeNormalUnitVector;
	
	private Face3d faceGeometry;
	
	private Vector3d testPoint;
	
	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private PointToFaceLocationKind pointToFaceLocationKind;
	
	//end: algorithm result arguments
	
	public PointToFaceLocationAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public IPointToFaceLocationResult calculate(Vector3d planeNormalUnitVector, Face3d face, Vector3d testPoint)
	{
		if (planeNormalUnitVector == null)
			throw new IllegalArgumentException("Plane normal unit vector is not defined.");
		if (face == null)
			throw new IllegalArgumentException("Face is not defined.");
		if (testPoint == null)
			throw new IllegalArgumentException("Test point is not defined");
		
		this.planeNormalUnitVector = planeNormalUnitVector;
		this.faceGeometry = face;
		this.testPoint = testPoint;
		
		return super.calculate();
	}

	@Override
	protected void calculateInternal()
	{
		Polygon3d[] facePolygons = this.faceGeometry.polygons();
		for (int i = 0; i < facePolygons.length; i++)
		{
			IPointToPolygonLocationAlgorithm pointToPolygonAlgorithm = new PointToPolygonLocationAlgorithm(this.precisionConfiguration);
			IPointToPolygonLocationResult result = pointToPolygonAlgorithm.calculate(this.planeNormalUnitVector, facePolygons[i], this.testPoint);
			
			if (result.hasErrors())
			{
				this.addErrors(result.errors());
				return;
			}
			
			try
			{
				PointToPolygonLocationKind pointToPolygonLocationKind = result.locationKind();
				if (pointToPolygonLocationKind != PointToPolygonLocationKind.Inside)
				{
					if (pointToPolygonLocationKind == PointToPolygonLocationKind.OnBoundary)
					{
						this.pointToFaceLocationKind = PointToFaceLocationKind.EdgeInterior;
						return;
					} 
					else if (pointToPolygonLocationKind == PointToPolygonLocationKind.Outside)
					{
						this.pointToFaceLocationKind = PointToFaceLocationKind.Exterior;
						return;
					}
				}
			}
			catch (Exception e)
			{
				this.addError(e.getMessage());
				return;
			}
		}
		this.pointToFaceLocationKind = PointToFaceLocationKind.Interior;
	}

	@Override
	protected IPointToFaceLocationResult constructResult()
	{
		return new PointToFaceLocationResult(this.pointToFaceLocationKind);
	}

	@Override
	protected IPointToFaceLocationResult constructErrorsResult()
	{
		return new PointToFaceLocationResult(this.errors());
	}

}
