package spatial.algorithms.pointToLineLocation;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.geometry.Line3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToLineLocation.IPointToLineLocationAlgorithm;
import spatial.model.algorithms.pointToLineLocation.IPointToLineLocationResult;
import spatial.model.algorithms.pointToLineLocation.PointToLineLocationKind;

public class PointToLineLocationAlgorithm extends AlgorithmBase<IPointToLineLocationResult> implements IPointToLineLocationAlgorithm
{
	//start: algorithm input arguments
	
	private Vector3d planeNormalUnitVector;
	
	private Line3d line;
	
	private Vector3d testPoint;
	
	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private PointToLineLocationKind locationKind;
	
	//end: algorithm result arguments
	
	public PointToLineLocationAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public IPointToLineLocationResult calculate(Vector3d planeNormalUnitVector, Line3d line, Vector3d testPoint)
	{
		if (planeNormalUnitVector == null)
			throw new IllegalArgumentException("The Plane Normal Vector is not defined.");
		if (line == null)
			throw new IllegalArgumentException("The Line is not defined.");
		if (testPoint == null)
			throw new IllegalArgumentException("The Test Point is not defined.");
		
		this.planeNormalUnitVector = planeNormalUnitVector;
		this.line = line;
		this.testPoint = testPoint;
		
		return super.calculate();
	}

	@Override
	protected void calculateInternal()
	{
		Vector3d t = this.testPoint.substract(line.linePoint(), this.precisionConfiguration);
		
		Vector3d x = this.line.lineUnitVector().crossProduct(t, this.precisionConfiguration);
		double angle = this.planeNormalUnitVector.dotProduct(x);
		
		if (this.precisionConfiguration.compareWithAccuracy(angle, 0) == 0)
		{
			this.locationKind = PointToLineLocationKind.Line;
//			//make more precise calculation
//			Vector3d nextPointOnTheLine = this.line.linePoint().add(this.line.lineUnitVector().mul(t.euqlideanNorm(), this.precisionConfiguration), this.precisionConfiguration);
//			Edge3d edgeOnTheLine = new Edge3d(this.line.linePoint(), nextPointOnTheLine);
//			
//			IPointToEdgeDistanceAlgorithm pointToEdgeDistanceAlgorithm = new PointToEdgeDistanceAlgorithm(this.precisionConfiguration);
//			IPointToEdgeDistanceResult pointToEdgeDistanceResult = pointToEdgeDistanceAlgorithm.calculate(edgeOnTheLine, this.testPoint);
//			
//			if (pointToEdgeDistanceResult.hasErrors())
//			{
//				this.addErrors(pointToEdgeDistanceResult.errors());
//				return;
//			}
//			else
//			{
//				Vector3d orthogonalDistanceVector = pointToEdgeDistanceResult.projectionPoint().substract(this.testPoint, this.precisionConfiguration);
//				double orthogonalDistance = orthogonalDistanceVector.euqlideanNorm();
//				
//				if (this.precisionConfiguration.compareWithAccuracy(orthogonalDistance, 0) == 0)
//				{
//					this.locationKind = PointToLineLocationKind.Line;
//				}
//				else
//				{
//					//call the algorithm recursively for corrected result
//					PointToLineLocationAlgorithm recursiveAlgorithm = new PointToLineLocationAlgorithm(this.precisionConfiguration);
//					IPointToLineLocationResult recursiveResult = recursiveAlgorithm.calculate(this.planeNormalUnitVector, this.line, this.testPoint);
//					
//					if (recursiveResult.hasErrors())
//						this.addErrors(recursiveResult.errors());
//					else
//						this.locationKind = recursiveResult.locationKind();
//				}
//			}
		}
		else if (this.precisionConfiguration.compareWithAccuracy(angle, 0) == -1)
		{
			this.locationKind = PointToLineLocationKind.Right;
		}
		else
		{
			this.locationKind = PointToLineLocationKind.Left;
		}
	}

	@Override
	protected IPointToLineLocationResult constructResult()
	{
		return new PointToLineLocationResult(this.locationKind);
	}

	@Override
	protected IPointToLineLocationResult constructErrorsResult()
	{
		return new PointToLineLocationResult(this.errors());
	}

}
