package spatial.algorithms.pointToPlaneDistance;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.geometry.Plane3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToPlaneDistance.IPointToPlaneDistanceAlgorithm;
import spatial.model.algorithms.pointToPlaneDistance.IPointToPlaneDistanceResult;

public class PointToPlaneDistanceAlgorithm extends AlgorithmBase<IPointToPlaneDistanceResult> implements IPointToPlaneDistanceAlgorithm
{
	//start: algorithm input arguments

	private Vector3d testPoint;
	
	private Plane3d plane;

	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private double distance;
	
	private Vector3d projectionPoint;
	
	//end: algorithm result arguments
	
	public PointToPlaneDistanceAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public IPointToPlaneDistanceResult calculate(Plane3d plane, Vector3d testPoint)
	{
		if (plane == null)
			throw new IllegalArgumentException("The Plane is not defined.");
		if (testPoint == null)
			throw new IllegalArgumentException("The Test Point is not defined.");
		
		this.plane = plane;
		this.testPoint = testPoint;
		
		return super.calculate();
	}

	@Override
	protected void calculateInternal()
	{
		Vector3d x0tVector = this.testPoint.substract(this.plane.pointOnThePlane(), this.precisionConfiguration);
		
		this.distance = this.plane.normalUnitVector().dotProduct(x0tVector);
		
		Vector3d x0ToProjectionPointVector = x0tVector.substract(this.plane.normalUnitVector().mul(this.distance, this.precisionConfiguration), this.precisionConfiguration);
		this.projectionPoint = this.plane.pointOnThePlane().add(x0ToProjectionPointVector, this.precisionConfiguration);
	}

	@Override
	protected IPointToPlaneDistanceResult constructResult()
	{
		return new PointToPlaneDistanceResult(this.distance, this.projectionPoint);
	}

	@Override
	protected IPointToPlaneDistanceResult constructErrorsResult()
	{
		return new PointToPlaneDistanceResult(this.errors());
	}

	
}
