package spatial.algorithms.pointToPolygonDistance;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.algorithms.pointToEdgeDistance.PointToEdgeDistanceAlgorithm;
import spatial.geometry.Edge3d;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToEdgeDistance.IPointToEdgeDistanceAlgorithm;
import spatial.model.algorithms.pointToEdgeDistance.IPointToEdgeDistanceResult;
import spatial.model.algorithms.pointToEdgeDistance.PointToEdgeLocationKind;
import spatial.model.algorithms.pointToPolygonDistance.IPointToPolygonDistanceAlgorithm;
import spatial.model.algorithms.pointToPolygonDistance.IPointToPolygonDistanceResult;

public final class PointToPolygonDistanceAlgorithm extends AlgorithmBase<IPointToPolygonDistanceResult> implements IPointToPolygonDistanceAlgorithm
{
	//start: algorithm input arguments
	
	@SuppressWarnings("unused")
	private Vector3d planeNormalUnitVector;
	
	private Polygon3d polygon;
	
	private Vector3d testPoint;

	//end: algorithm input arguments
	
	//start: algorithm result arguments
	
	private double distance;
	
	private Vector3d polygonDistancePoint;
	
	private int edgeIndex;
	
	private PointToEdgeLocationKind distancePointLocationKind;
	
	private Vector3d polygonDistanceProjectionPoint;
	
	//end: algorithm result arguments
	
	public PointToPolygonDistanceAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public IPointToPolygonDistanceResult calculate(Vector3d planeNormalUnitVector, Polygon3d polygon, Vector3d testPoint)
	{
		if (planeNormalUnitVector == null)
			throw new IllegalArgumentException("The Plane Normal Unit Vector is not defined.");
		if (polygon == null)
			throw new IllegalArgumentException("The Polygon is not defined.");
		if (testPoint == null)
			throw new IllegalArgumentException("The Test Point is not defined.");
		
		this.planeNormalUnitVector = planeNormalUnitVector;
		this.polygon = polygon;
		this.testPoint = testPoint;
		
		return super.calculate();
	}

	@Override
	protected void calculateInternal()
	{
		//algorithm
		int nearestEdgeIx = -1;
		IPointToEdgeDistanceResult minPointToEdgeDistanceResult = null;
		
		for (int i = 0; i < this.polygon.vertices().length; i++)
		{
			Edge3d edge = this.polygon.getEdge(i);
			
			IPointToEdgeDistanceAlgorithm pointToEdgeDistanceAlgorithm = new PointToEdgeDistanceAlgorithm(this.precisionConfiguration);
			IPointToEdgeDistanceResult result = pointToEdgeDistanceAlgorithm.calculate(edge, this.testPoint);
						
			if (result.hasErrors())
			{
				this.addErrors(result.errors());
				return;
			}

			if (nearestEdgeIx == -1 || minPointToEdgeDistanceResult.distance() > result.distance())
			{
				nearestEdgeIx = i;
				minPointToEdgeDistanceResult = result;
			}
		}

		//forming the result of the algorithm
		this.edgeIndex = nearestEdgeIx;
		this.distance = minPointToEdgeDistanceResult.distance();
		this.distancePointLocationKind = minPointToEdgeDistanceResult.distancePointLocationKind();
		this.polygonDistancePoint = minPointToEdgeDistanceResult.edgeDistancePoint();
		this.polygonDistanceProjectionPoint = minPointToEdgeDistanceResult.projectionPoint();
	}

	@Override
	protected IPointToPolygonDistanceResult constructResult()
	{
		return new PointToPolygonDistanceResult(this.distance, this.polygonDistancePoint, this.edgeIndex, this.distancePointLocationKind, this.polygonDistanceProjectionPoint);
	}

	@Override
	protected IPointToPolygonDistanceResult constructErrorsResult()
	{
		return new PointToPolygonDistanceResult(this.errors());
	}
}
