package spatial.algorithms.pointToPolygonDistance;

import spatial.algorithms.AlgorithmResultBase;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToEdgeDistance.PointToEdgeLocationKind;
import spatial.model.algorithms.pointToPolygonDistance.IPointToPolygonDistanceResult;

final class PointToPolygonDistanceResult extends AlgorithmResultBase implements IPointToPolygonDistanceResult
{
	private final double distance;

	private final Vector3d polygonDistancePoint;
	
	private final Vector3d polygonDistanceProjectionPoint;
	
	private final int edgeIndex;
	
	private final PointToEdgeLocationKind distancePointLocationKind;
	
	public PointToPolygonDistanceResult(double distance, Vector3d polygonDistancePoint, int edgeIndex, PointToEdgeLocationKind distancePointLocationKind, Vector3d polygonDistanceProjectionPoint)
	{
		if (polygonDistancePoint == null)
			throw new IllegalArgumentException("Polygon Distance Point is not defined.");
		if (distancePointLocationKind == null)
			throw new IllegalArgumentException("Distance Point Location Kind is not defined.");
		if (polygonDistanceProjectionPoint == null)
			throw new IllegalArgumentException("Distance Point Distance Projection Point is not defined.");
		
		this.distance = distance;
		this.polygonDistancePoint = polygonDistancePoint;
		this.edgeIndex = edgeIndex;
		this.distancePointLocationKind = distancePointLocationKind;
		this.polygonDistanceProjectionPoint = polygonDistanceProjectionPoint;
	}

	public PointToPolygonDistanceResult(String[] errors)
	{
		super(errors);

		this.distance = 0;
		this.polygonDistancePoint = null;
		this.edgeIndex = -1;
		this.distancePointLocationKind = PointToEdgeLocationKind.NotSet;
		this.polygonDistanceProjectionPoint = null;
	}

	@Override
	public double distance()
	{
		return this.distance;
	}

	@Override
	public Vector3d polygonDistancePoint()
	{
		return this.polygonDistancePoint;
	}
	
	@Override
	public Vector3d polygonDistanceProjectionPoint()
	{
		return this.polygonDistanceProjectionPoint;
	}

	@Override
	public int edgeIndex()
	{
		return this.edgeIndex;
	}

	@Override
	public PointToEdgeLocationKind distancePointLocationKind()
	{
		return this.distancePointLocationKind;
	}

}
