package spatial.algorithms.polygonArea;

import precision.model.IPrecisionConfiguration;
import spatial.algorithms.AlgorithmBase;
import spatial.geometry.Polygon2d;
import spatial.geometry.Vector2d;
import spatial.model.algorithms.polygonArea.IPolygonAreaAlgorithm;
import spatial.model.algorithms.polygonArea.IPolygonAreaResult;

public final class PolygonAreaAlgorithm extends AlgorithmBase<IPolygonAreaResult> implements IPolygonAreaAlgorithm
{

	public PolygonAreaAlgorithm(IPrecisionConfiguration precisionConfiguration)
	{
		super(precisionConfiguration);
	}

	@Override
	public IPolygonAreaResult calculate(Polygon2d polygon)
	{
		double sum = 0;
		
		Vector2d[] vertices = polygon.vertices();
		for (int i = 0; i < vertices.length; i++)
		{
			Vector2d s = vertices[i];
			Vector2d e = vertices[(i + 1) % vertices.length];
			
			sum += (e.x() + s.x()) * (e.y() - s.y()) - (e.y() + s.y()) * (e.x() - s.x());
		}
		
		return new PolygonAreaResult(sum / 4.0);
	}

	@Override
	protected void calculateInternal()
	{
		
	}

	@Override
	protected IPolygonAreaResult constructResult()
	{
		return null;
	}

	@Override
	protected IPolygonAreaResult constructErrorsResult()
	{
		return null;
	}

}
