package spatial.geometry;

import precision.model.IPrecisionConfiguration;
import spatial.model.IGeometry;

/**
 * The line in 3d space.
 *
 */
public final class Line3d implements IGeometry 
{
	private final Vector3d lineUnitVector;
	
	private final Vector3d linePoint;
	
	public Line3d(Vector3d lineUnitVector, Vector3d linePoint, IPrecisionConfiguration precisionConfiguration) throws IllegalArgumentException
	{
		if (lineUnitVector == null)
			throw new IllegalArgumentException("lineUnitVector");
		if (linePoint == null)
			throw new IllegalArgumentException("linePoint");
		if (!lineUnitVector.isUnit(precisionConfiguration))
			throw new IllegalArgumentException("lineUnitVector");
		
		this.lineUnitVector = lineUnitVector;
		this.linePoint = linePoint;
	}
	
	/**
	 * The unit vector to define line direction;
	 */
	public final Vector3d lineUnitVector()
	{
		return this.lineUnitVector;
	}
	
	/**
	 * The arbitrary point on the line.
	 */
	public final Vector3d linePoint()
	{
		return this.linePoint;
	}
}
