package spatial.geometry;

import precision.model.IPrecisionConfiguration;

public final class Polygon3d extends PolygonXd<Vector3d, Edge3d>
{
	private ClockWiseType clockWiseType;
	
	/**
	 * Constructor.
	 * @param vertices Polygon vertices.
	 */
	public Polygon3d(Vector3d[] vertices)
	{
		super(vertices);
		
		this.clockWiseType = ClockWiseType.NotSet;
	}

	@Override
	protected final spatial.geometry.Edge3d newE(spatial.geometry.Vector3d a, spatial.geometry.Vector3d b)
	{
		// TODO Auto-generated method stub
		return new Edge3d (a, b);
	}

	public ClockWiseType clockWiseType(Vector3d planeNormal, IPrecisionConfiguration precisionConfiguration)
	{
		if (this.clockWiseType == ClockWiseType.NotSet)
		{
			Vector3d cpSum = new Vector3d(0, 0, 0, precisionConfiguration);
			for (int i = 0; i < this.vertices().length; i++)
			{
				Vector3d e1 = this.vertices()[(i + 1) % this.vertices().length].substract(this.vertices()[i], precisionConfiguration);
				Vector3d e2 = this.vertices()[(i + 2) % this.vertices().length].substract(this.vertices()[(i + 1) % this.vertices().length], precisionConfiguration);
				
				cpSum = cpSum.add(e1.crossProduct(e2, precisionConfiguration), precisionConfiguration);
			}
			
			cpSum = cpSum.unitVector(precisionConfiguration);
			
			double dp = planeNormal.dotProduct(cpSum);
			
			if (precisionConfiguration.compareWithAccuracy(dp, 1) == 0)
			{
				this.clockWiseType = ClockWiseType.CounterClockWise;
			} 
			else if (precisionConfiguration.compareWithAccuracy(dp, -1) == 0)
			{
				this.clockWiseType = ClockWiseType.ClockWise;
			}
			else
			{
				this.clockWiseType = ClockWiseType.NotApplicable;
			}
		}
		
		return this.clockWiseType;
	}
	
	
}
