package spatial.geometry;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import spatial.model.IGeometry;

abstract class PolygonXd<V, E> implements IGeometry
{
	private final V[] vertices;

	/**
	 * Constructor.
	 * @param vertices Polygon vertices.
	 */
	protected PolygonXd(V[] vertices)
	{
		if (vertices == null)
			throw new IllegalArgumentException("Polygon vertices are not defined.");
		if (vertices.length < 3)
			throw new IllegalArgumentException("Defined less than 3 vertices for polygon.");

		this.vertices = vertices;
	}
	
	public final V[] vertices()
	{
		return this.vertices;
	}
	
	public final Enumeration<E> edgesEnumerator()
	{
		return new EdgesEnumerator(this.vertices, this);
	}
	
	/**
	 * Returns the edge of the polygon by its index.
	 */
	public final E getEdge(int index)
	{
		if (index >= this.vertices.length || index < 0)
			throw new NoSuchElementException();
		return this.newE(this.vertices[index++], this.vertices[index % this.vertices.length]);
	}
	
	protected abstract E newE(V a, V b);

	class EdgesEnumerator implements Enumeration<E>
	{
		private V[] vertices;
		
		private int index;
		
		private PolygonXd<V, E> efvc;
		
		public EdgesEnumerator(V[] vertices, PolygonXd<V, E> efvc)
		{
			if (vertices == null)
				throw new IllegalArgumentException("Polygon is not defined.");
			this.vertices = vertices;
			this.index = 0;
			this.efvc = efvc;
		}
		
		@Override
		public boolean hasMoreElements()
		{
			return this.index < this.vertices.length;
		}

		@Override
		public E nextElement()
		{
			if (this.index == this.vertices.length)
				throw new NoSuchElementException();
			return this.efvc.newE(this.vertices[this.index++], this.vertices[this.index % this.vertices.length]);
		}
	}
}