package spatial.model.algorithms.csTransformation;

import spatial.geometry.Vector3d;

/**
 * Local Point Coordinates in an Oriented Plane
 */
public interface ICsTransformationAlgorithm
{
	/**
	 * Determine a local coordinate system in a plane and compute the local coordinates of a set of points, whose global coordinates are given.
	 * @param normalPlaneOrt The unit normal vector n of the plane (x3).
	 * @param inPlaneOrt The unit vector u of the plane (in-plane x1).
	 * @param localCSOrigin The Local Coordinate System reference point in the plane.
	 * @param globalCsPoints The set of points with coordinates in the global CS. 
	 */
	ICsTransformationResult calculate(Vector3d normalPlaneOrt, Vector3d inPlaneOrt, Vector3d localCSOrigin, Vector3d[] globalCsPoints);
}
