package spatial.model.algorithms.edgeToFaceLocation;

import spatial.geometry.Edge3d;
import spatial.geometry.Face3d;
import spatial.geometry.Vector3d;

/**
 * Location of an Edge Relative to an Oriented Face.
 */
public interface IEdgeToFaceLocationAlgorithm
{
	/**
	 * A plane with unit normal vector n contains an oriented face and a test edge E with nodes A and B. 
	 * The face is defined by an outer polygon and an optional number of inner polygons. 
	 * The location of edge E relative to the face is to be determined.
	 * @param planeNormalUnitVector Plane normal unit vector.
	 * @param face The face.
	 * @param testEdge The test edge.
	 */
	IEdgeToFaceLocationResult calculate(Vector3d planeNormalUnitVector, Face3d face, Edge3d testEdge);
}
