package spatial.model.elements.set;

import java.util.Arrays;

import spatial.model.IElement;
import spatial.model.IGeometry;
import spatial.model.elements.set.exception.ElementWithTheSameGeometryAlreadyExistsException;
import spatial.model.elements.set.exception.ElementWithTheSameNameAlreadyExistsException;

/**
 * The collection of the collided by hash elements.
 */
final class CollidedElementsContainer
{
	private int key;
	
	private IElement[] elements;
	
	/**
	 * The actual number of elements regardless to the length of elements array.
	 */
	private int size;
	
	public CollidedElementsContainer(int key, IElement element) 
	{
		this.key = key;
		this.elements = new IElement[1];
		
		this.elements[0] = element;
		this.size = 1;
	}
	
	public int key()
	{
		return this.key;
	}
	
	public IElement[] elements()
	{
		return this.elements;
	}
	
	public void addElement(IElement element) throws ElementWithTheSameNameAlreadyExistsException, ElementWithTheSameGeometryAlreadyExistsException
	{
		//Check cases:
		//1. basketElement.Name == element.Name AND basketElement.Geometry == element.Geometry => return
		//2. basketElement.Name == element.Name AND basketElement.Geometry != element.Geometry => ElementWithTheSameNameAlreadyExistsException
		//3. basketElement.Name != element.Name AND basketElement.Geometry == element.Geometry => ElementWithTheSameGeometryAlreadyExistsException
		//4. basketElement.Name != element.Name AND basketElement.Geometry != element.Geometry => add element to collection && return
		for (int i = 0; i < this.elements.length; i++)
		{
			IElement collectionElement = this.elements[i];
			
			if (collectionElement.name().equals(element.name()))
			{
				if (!collectionElement.geometry().equals(element.geometry()))
				{
					//case 2:
					//the exception for this case throws by <T>Set before this code could be executed
					throw new ElementWithTheSameNameAlreadyExistsException(element.name(), collectionElement.toString(), element.toString());
				}
				else
				{
					//case 1:
					return;
				}
			}
			else
			{
				if (collectionElement.geometry().equals(element.geometry()))
				{
					//case 3:
					throw new ElementWithTheSameGeometryAlreadyExistsException(element.geometry().toString(), collectionElement.name(), element.name());
				} 
				//else
					//case 4:
					//element will be added to collection
			}
		}
		
		//now we can add node to the basket
		if (this.size == this.elements.length)
		{
			IElement[] newElements = Arrays.copyOf(this.elements, this.elements.length + 1);
			this.elements = newElements;
		}
		this.elements[size] = element;
		this.size++;
	}
	
	public IElement get(IGeometry geometry) 
	{
		int i = 0;
		while (i < this.elements.length)
		{
			if (this.elements[i].geometry().equals(geometry))
				return this.elements[i];
			i++;
		}				
		return null;
	}

	public void remove(IGeometry geometry)
	{
		int i = 0;
		while (i < this.elements.length && !this.elements[i].geometry().equals(geometry))
			i++;
	
		if (i != this.elements.length && this.elements[i].geometry().equals(geometry))
		{
			for (int j = i + 1; j < this.elements.length; j++)
			{
				this.elements[j - 1] = this.elements[j];
			}
			
			if (size > 0)
				size--;
		}
	}
}
