package spatial.model.elements.set;

import spatial.model.IElement;
import spatial.model.IGeometry;
import spatial.model.elements.set.exception.ElementGeometryOfNotValidTypeException;
import spatial.model.elements.set.exception.ElementWithTheSameGeometryAlreadyExistsException;
import spatial.model.elements.set.exception.ElementWithTheSameNameAlreadyExistsException;

/**
 * The interface for the any element set in the Spatial Model.
 */
public interface IElementSet
{
	/**
	 * Add or retrieve existed element from the set.
	 * @return The add element or existed element of the Set if the add element equals to some element in the Set.
	 */
	IElement ensure(IElement element) 
			throws ElementWithTheSameNameAlreadyExistsException, 
			ElementWithTheSameGeometryAlreadyExistsException, 
			ElementGeometryOfNotValidTypeException;
	
	/**
	 * Test the set contains the element with the certain name.
	 */
	boolean contains(String name);
	
	/**
	 * Get the element from the set by its name.
	 */
	IElement get(String name);
	
	/**
	 * Get the element from the set by its geometry.
	 */
	IElement get(IGeometry geometry) throws ElementGeometryOfNotValidTypeException;
	
	/**
	 * Remove element from the set by its name.
	 */
	void remove(String name);
	
}
