package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.edgeToFaceLocation.EdgeToFaceLocationAlgorithm;
import spatial.geometry.Edge3d;
import spatial.geometry.Face3d;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.edgeToFaceLocation.EdgeToFaceLocationKind;
import spatial.model.algorithms.edgeToFaceLocation.IEdgeToFaceLocationResult;

class EdgeToFaceLocationFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}
	
	@Test
	void testCalculateVector3dFace3dEdge3d()
	{
		Vector3d planeNormalUnitVector = new Vector3d(0, 0, 1, precisionConfiguration);
		Face3d face = new Face3d(
				new Polygon3d[] 
						{
								new Polygon3d(new Vector3d[] {
										new Vector3d(12.5, 12.5, 0, precisionConfiguration),
										new Vector3d(62.5, 12.5, 0, precisionConfiguration),
										new Vector3d(62.5, 50, 0, precisionConfiguration),
										new Vector3d(12.5, 50, 0, precisionConfiguration)
								}),
								
								new Polygon3d(new Vector3d[] {
										new Vector3d(50, 42.5, 0, precisionConfiguration),
										new Vector3d(50, 25, 0, precisionConfiguration),
										new Vector3d(25, 25, 0, precisionConfiguration),
										new Vector3d(37.5, 37.5, 0, precisionConfiguration),
										new Vector3d(37.5, 37.5, 0, precisionConfiguration)
								})
				});
		
		Edge3d[] innerEdges = new Edge3d[] {
				new Edge3d(new Vector3d(17.5, 42.5, 0, precisionConfiguration), new Vector3d(22.5, 47.5, 0, precisionConfiguration))
		};
		
		Edge3d[] outerEdges = new Edge3d[] {
				new Edge3d(new Vector3d(34.5, 30, 0, precisionConfiguration), new Vector3d(39.5, 35, 0, precisionConfiguration))
		};
		
		for (int i = 0; i < innerEdges.length; i++)
		{
			IEdgeToFaceLocationResult result = (new EdgeToFaceLocationAlgorithm(precisionConfiguration)).calculate(planeNormalUnitVector, face, innerEdges[i]);
			
			assertNull(result.errors());
			assertEquals(EdgeToFaceLocationKind.Interior, result.locationKind());
		}
		
		for (int i = 0; i < outerEdges.length; i++)
		{
			IEdgeToFaceLocationResult result = (new EdgeToFaceLocationAlgorithm(precisionConfiguration)).calculate(planeNormalUnitVector, face, outerEdges[i]);
			
			assertNull(result.errors());
			assertEquals(EdgeToFaceLocationKind.Exterior, result.locationKind());
		}
	}

}
