package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.edgeToPolygonLocation.EdgeToPolygonLocationAlgorithm;
import spatial.geometry.Edge3d;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.edgeToPolygonLocation.EdgeToPolygonLocationKind;
import spatial.model.algorithms.edgeToPolygonLocation.IEdgeToPolygonLocationAlgorithm;
import spatial.model.algorithms.edgeToPolygonLocation.IEdgeToPolygonLocationResult;

class EdgeToPolygonLocationFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}
	
	Polygon3d getPolygon()
	{
		return new Polygon3d(new Vector3d[] {
						new Vector3d(5, 5, 0, precisionConfiguration),
						new Vector3d(25, 5, 0, precisionConfiguration),
						new Vector3d(17.5, 10, 0, precisionConfiguration),
						new Vector3d(20, 17.5, 0, precisionConfiguration),
						new Vector3d(25, 22.5, 0, precisionConfiguration),
						new Vector3d(12.5, 17.5, 0, precisionConfiguration),
						new Vector3d(5, 22.5, 0, precisionConfiguration)
				});
	}

	@Test
	void testCalculate01()
	{
		//test in conditions
		
		//arrange
		Polygon3d polygon = this.getPolygon();
		Edge3d[] testEdges = new Edge3d[]
				{
					new Edge3d(new Vector3d(7.5, 12.5, 0, precisionConfiguration), new Vector3d(12.5, 12.5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(5, 5, 0, precisionConfiguration), new Vector3d(10, 7.5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(10, 7.5, 0, precisionConfiguration), new Vector3d(5, 5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(21.25, 7.5, 0, precisionConfiguration), new Vector3d(15, 5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(17.5, 10, 0, precisionConfiguration), new Vector3d(12.5, 17.5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(17.5000, 12.5000, 0, precisionConfiguration), new Vector3d(17.5000, 7.5000, 0, precisionConfiguration))
				};
		Vector3d planeNormalUnitVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		//act
		IEdgeToPolygonLocationResult[] results = new IEdgeToPolygonLocationResult[testEdges.length];
		for (int i = 0; i < testEdges.length; i++)
		{
			IEdgeToPolygonLocationAlgorithm algorithm = new EdgeToPolygonLocationAlgorithm(precisionConfiguration);
			IEdgeToPolygonLocationResult result = algorithm.calculate(planeNormalUnitVector, polygon, testEdges[i]);
			results[i] = result;
		}
		
		//assert
		for (int i = 0; i < results.length; i++)
		{
			assertEquals(EdgeToPolygonLocationKind.Interior, results[i].locationKind(), "Not expected result for " + i);
		}
	}
	
	@Test
	void testCalculate02()
	{
		//test outer conditions
		
		//arrange
		Polygon3d polygon = this.getPolygon();
		Edge3d[] testEdges = new Edge3d[]
				{
					new Edge3d(new Vector3d(7.5, 25, 0, precisionConfiguration), new Vector3d(12.5, 25, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(8.75, 20, 0, precisionConfiguration), new Vector3d(12.5, 22.5, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(12.5, 22.5, 0, precisionConfiguration), new Vector3d(8.75, 20, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(22.5, 20, 0, precisionConfiguration), new Vector3d(25, 5, 0, precisionConfiguration))
				};
		Vector3d planeNormalUnitVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		//act
		IEdgeToPolygonLocationResult[] results = new IEdgeToPolygonLocationResult[testEdges.length];
		for (int i = 0; i < testEdges.length; i++)
		{
			IEdgeToPolygonLocationAlgorithm algorithm = new EdgeToPolygonLocationAlgorithm(precisionConfiguration);
			IEdgeToPolygonLocationResult result = algorithm.calculate(planeNormalUnitVector, polygon, testEdges[i]);
			results[i] = result;
		}
		
		//assert
		for (int i = 0; i < results.length; i++)
		{
			assertEquals(EdgeToPolygonLocationKind.Exterior, results[i].locationKind(), "Not expected result for " + i);
		}
	}
	
	@Test
	void testCalculate03()
	{
		//test error conditions
		
		//arrange
		Polygon3d polygon = this.getPolygon();
		Edge3d[] testEdges = new Edge3d[]
				{
					new Edge3d(new Vector3d(2.5, 15, 0, precisionConfiguration), new Vector3d(7.5, 10, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(18.7500, 20.0000, 0, precisionConfiguration), new Vector3d(21.0344828, 20.9137931, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(17.5000, 16.5000, 0, precisionConfiguration), new Vector3d(19.0000, 7.5000, 0, precisionConfiguration)),
					new Edge3d(new Vector3d(22.5000, 20.0000, 0, precisionConfiguration), new Vector3d(19.0000, 16.5000, 0, precisionConfiguration))
				};
		Vector3d planeNormalUnitVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		//act
		IEdgeToPolygonLocationResult[] results = new IEdgeToPolygonLocationResult[testEdges.length];
		for (int i = 0; i < testEdges.length; i++)
		{
			IEdgeToPolygonLocationAlgorithm algorithm = new EdgeToPolygonLocationAlgorithm(precisionConfiguration);
			IEdgeToPolygonLocationResult result = algorithm.calculate(planeNormalUnitVector, polygon, testEdges[i]);
			results[i] = result;
		}
		
		//assert
		for (int i = 0; i < results.length; i++)
		{
			assertEquals(EdgeToPolygonLocationKind.Error, results[i].locationKind(), "Not expected result for " + i);
		}
	}

}
