package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import precision.PreciseDoubleFactory;
import precision.PrecisionConfiguration;
import precision.model.IPreciseDoubleFactory;
import precision.model.IPrecisionConfiguration;
import spatial.NodeFactory;
import spatial.geometry.Vector3d;
import spatial.model.IGeometry;
import spatial.model.elements.INode;

class NodeFixture 
{
	private IPrecisionConfiguration precisionConfiguration;
	private IPreciseDoubleFactory preciseDoubleFactory;
	
	@BeforeEach
	void setUpBeforeEach() throws Exception 
	{
		this.precisionConfiguration = new PrecisionConfiguration(4, 0.00009);
		this.preciseDoubleFactory = new PreciseDoubleFactory(this.precisionConfiguration);
	}
	
	@Test
	void testGetName()
	{
		INode node = (new NodeFactory()).create("nodeA", new Vector3d(10, 20, 30, this.preciseDoubleFactory));

		assertEquals("nodeA", node.name());
	}

	@Test
	void testGetGeometry() 
	{
		INode node = (new NodeFactory()).create("nodeA", new Vector3d(10, 20, 30, this.preciseDoubleFactory));

		assertEquals((IGeometry)new Vector3d(10, 20, 30, this.preciseDoubleFactory), node.geometry());
	}
	
	@Test
	void testHashCode() 
	{
		int n1 = (new NodeFactory()).create("nodeA", new Vector3d(10, 20, 30, this.preciseDoubleFactory)).hashCode();
		int n2 = (new NodeFactory()).create("nodeA", new Vector3d(10, 10, 10, this.preciseDoubleFactory)).hashCode();
		int n3 = (new NodeFactory()).create("nodeB", new Vector3d(10, 10, 10, this.preciseDoubleFactory)).hashCode();
		
		assertNotEquals(n1, n2);
		assertNotEquals(n1, n3);
		assertNotEquals(n2, n3);
	}

	@Test
	void testEqualsObject() 
	{
		INode nodeA = (new NodeFactory()).create("nodeA", new Vector3d(10, 20, 30, this.preciseDoubleFactory));
		INode nodeDvA = (new NodeFactory()).create("nodeA", new Vector3d(10, 20, 30, this.preciseDoubleFactory));

		assertEquals(nodeA, nodeDvA);
	}

}
