package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.pointToFaceLocation.PointToFaceLocationAlgorithm;
import spatial.geometry.Face3d;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToFaceLocation.IPointToFaceLocationAlgorithm;
import spatial.model.algorithms.pointToFaceLocation.IPointToFaceLocationResult;
import spatial.model.algorithms.pointToFaceLocation.PointToFaceLocationKind;

class PointToFaceLocationFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}
	
	Face3d getTestFace()
	{
		return new Face3d(new Polygon3d[] {
				new Polygon3d(new Vector3d[] {
						new Vector3d(10, 10, 0, precisionConfiguration),
						new Vector3d(30, 10, 0, precisionConfiguration),
						new Vector3d(37.5, 25, 0, precisionConfiguration),
						new Vector3d(20, 30, 0, precisionConfiguration),
						new Vector3d(20, 22.5, 0, precisionConfiguration),
						new Vector3d(15, 22.5, 0, precisionConfiguration),
						new Vector3d(15, 30, 0, precisionConfiguration),
						new Vector3d(10, 30, 0, precisionConfiguration)
				}),
				new Polygon3d(new Vector3d[] {
						new Vector3d(15, 12.5, 0, precisionConfiguration),
						new Vector3d(15, 20, 0, precisionConfiguration),
						new Vector3d(27.5, 12.5, 0, precisionConfiguration)
				}),
				new Polygon3d(new Vector3d[] {
						new Vector3d(16.5, 13.5, 0, precisionConfiguration),
						new Vector3d(22.5, 13.5, 0, precisionConfiguration),
						new Vector3d(16.5, 17, 0, precisionConfiguration)
				})
		});
	}

	@Test
	void testCalculate01()
	{
		//arrange
		Face3d face = this.getTestFace();		
		Vector3d testPoint = new Vector3d(33.7500, 17.5, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.EdgeInterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}

	@Test
	void testCalculate02()
	{
		//arrange
		Face3d face = this.getTestFace();		
		Vector3d testPoint = new Vector3d(21.2500, 16.2500, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.EdgeInterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate03()
	{
		//the test point inside inner polygon
		
		//arrange
		Face3d face = this.getTestFace();		
		Vector3d testPoint = new Vector3d(19.5000, 15.2500, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.EdgeInterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate04()
	{
		//arrange
		Face3d face = this.getTestFace();		
		Vector3d testPoint = new Vector3d(18, 26.5, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.Exterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate05()
	{
		//arrange
		Face3d face = this.getTestFace();
		Vector3d testPoint = new Vector3d(24.6, 24.1, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.Exterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate06()
	{
		//arrange
		Face3d face = this.getTestFace();
		Vector3d testPoint = new Vector3d(18.8300379, 31.5510452, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.Exterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate07()
	{
		//arrange
		Face3d face = this.getTestFace();
		Vector3d testPoint = new Vector3d(21.3255074, 28.2427348, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.Exterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate08()
	{
		//the test point at node		
		
		//arrange
		Face3d face = this.getTestFace();
		Vector3d testPoint = new Vector3d(20, 30, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.EdgeNode;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate09()
	{
		//arrange
		Face3d face = this.getTestFace();
		Vector3d testPoint = new Vector3d(23.5, 13, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToFaceLocationKind expectedLocationKind = PointToFaceLocationKind.Exterior;
		
		//act
		IPointToFaceLocationAlgorithm algorithm = new PointToFaceLocationAlgorithm(precisionConfiguration);
		IPointToFaceLocationResult result = algorithm.calculate(planeNormalVector, face, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
}
