package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.pointToLineLocation.PointToLineLocationAlgorithm;
import spatial.geometry.Line3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToLineLocation.IPointToLineLocationAlgorithm;
import spatial.model.algorithms.pointToLineLocation.IPointToLineLocationResult;
import spatial.model.algorithms.pointToLineLocation.PointToLineLocationKind;

class PointToLineLocationFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}

	@Test
	void testCalculate01()
	{
		// ---input---
		// plane normal: (0, 1, 0)
		// line: (10, 0, 15) - (0.894427, 0, 0.447214)
		// testPoint: (15, 0, 25)
		
		// ---expected output---
		// distance kind: Right
		
		//arrange
		Vector3d planeNormalVector = new Vector3d(0, 1, 0, precisionConfiguration);
		Vector3d linePoint = new Vector3d(10, 0, 15, precisionConfiguration);
		Vector3d lineVector = new Vector3d(0.894427, 0, 0.447214, precisionConfiguration);
		Line3d line = new Line3d(lineVector, linePoint, precisionConfiguration);
		Vector3d testPoint = new Vector3d(15, 0, 25, precisionConfiguration);
		
		//act
		IPointToLineLocationAlgorithm algorithm = new PointToLineLocationAlgorithm(precisionConfiguration);
		IPointToLineLocationResult result = algorithm.calculate(planeNormalVector, line, testPoint);
		
		//assert
		assertFalse(result.hasErrors());
		assertEquals(PointToLineLocationKind.Right, result.locationKind(), "Not expected Location Kind result.");
	}
	
	@Test
	void testCalculate02()
	{
		// ---input---
		// plane normal: (0, 1, 0)
		// line: (10, 0, 15) - (0.894427, 0, 0.447214)
		// testPoint: (45, 0, 30)
		
		// ---expected output---
		// distance kind: Left
		
		//arrange
		Vector3d planeNormalVector = new Vector3d(0, 1, 0, precisionConfiguration);
		Vector3d linePoint = new Vector3d(10, 0, 15, precisionConfiguration);
		Vector3d lineVector = new Vector3d(0.894427190999916, 0, 0.447213595499958, precisionConfiguration);
		Line3d line = new Line3d(lineVector, linePoint, precisionConfiguration);
		Vector3d testPoint = new Vector3d(45, 0, 30, precisionConfiguration);
		
		//act
		IPointToLineLocationAlgorithm algorithm = new PointToLineLocationAlgorithm(precisionConfiguration);
		IPointToLineLocationResult result = algorithm.calculate(planeNormalVector, line, testPoint);
		
		//assert
		assertFalse(result.hasErrors());
		assertEquals(PointToLineLocationKind.Left, result.locationKind(), "Not expected Location Kind result.");
	}
	
	@Test
	void testCalculate03()
	{
		// ---input---
		// plane normal: (0, 1, 0)
		// line: (10, 0, 15) - (0.894427, 0, 0.447214)
		// testPoint: (30, 0, 25)
		
		// ---expected output---
		// distance kind: Line
		
		//arrange
		Vector3d planeNormalVector = new Vector3d(0, 1, 0, precisionConfiguration);
		Vector3d linePoint = new Vector3d(10, 0, 15, precisionConfiguration);
		Vector3d lineVector = new Vector3d(0.894427, 0, 0.447214, precisionConfiguration);
		Line3d line = new Line3d(lineVector, linePoint, precisionConfiguration);
		Vector3d testPoint = new Vector3d(30, 0, 25, precisionConfiguration);
		
		//act
		IPointToLineLocationAlgorithm algorithm = new PointToLineLocationAlgorithm(precisionConfiguration);
		IPointToLineLocationResult result = algorithm.calculate(planeNormalVector, line, testPoint);
		
		//assert
		assertFalse(result.hasErrors());
		assertEquals(PointToLineLocationKind.Line, result.locationKind(), "Not expected Location Kind result.");
	}

}
