package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.pointToPlaneDistance.PointToPlaneDistanceAlgorithm;
import spatial.geometry.Plane3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToPlaneDistance.IPointToPlaneDistanceAlgorithm;
import spatial.model.algorithms.pointToPlaneDistance.IPointToPlaneDistanceResult;

class PointToPlaneDistanceFixture
{
	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}

	@Test
	void testCalculate()
	{
		//arrange
		Plane3d plane = new Plane3d(2, 3, 4, -2, precisionConfiguration);
		Vector3d testPoint = new Vector3d(7, 7, 7, precisionConfiguration);
		
		double expectedDistance = 11.3274;
		
		//act
		IPointToPlaneDistanceAlgorithm algorithm = new PointToPlaneDistanceAlgorithm(precisionConfiguration);
		IPointToPlaneDistanceResult result = algorithm.calculate(plane, testPoint);
		
		//assert algorithm has no errors
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		//assert distance
		assertEquals(precisionConfiguration.compareWithAccuracy(expectedDistance, result.distance()), 0, "Not expected distance result.");
		
		//assert distance point
		Vector3d distanceVector = testPoint.substract(result.projectionPoint(), precisionConfiguration);
		double distancePointLength = distanceVector.euqlideanNorm();
		assertEquals(precisionConfiguration.compareWithAccuracy(expectedDistance, distancePointLength), 0, "Not expected distance point.");
		
	}

}