package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.pointToPolygonDistance.PointToPolygonDistanceAlgorithm;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToEdgeDistance.PointToEdgeLocationKind;
import spatial.model.algorithms.pointToPolygonDistance.IPointToPolygonDistanceAlgorithm;
import spatial.model.algorithms.pointToPolygonDistance.IPointToPolygonDistanceResult;

class PointToPolygonDistanceFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}

	@Test
	void testCalculate01()
	{
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(22.5, 20, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(22.5, 17.5, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		double expectedDistance = 2.5;
		Vector3d expectedDistancePoint = new Vector3d(22.5, 20, 0, precisionConfiguration);
		PointToEdgeLocationKind expectedDistancePointLocationKind = PointToEdgeLocationKind.EndPoint;
		int expectedIndex = 2;
		
		//act
		IPointToPolygonDistanceAlgorithm algorithm = new PointToPolygonDistanceAlgorithm(precisionConfiguration);
		IPointToPolygonDistanceResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertTrue(precisionConfiguration.compareWithAccuracy(expectedDistance, result.distance()) == 0, "Not expected distance");
		assertEquals(expectedDistancePoint, result.polygonDistancePoint(), "Not expected distance point");		
		assertEquals(expectedDistancePointLocationKind, result.distancePointLocationKind(), "Not expected location kind.");
		assertEquals(expectedIndex, result.edgeIndex(), "Not expected Edge Index.");
	}

	@Test
	void testCalculate02()
	{
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(22.5, 20, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(20, 25, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		double expectedDistance = 3.3541;
		Vector3d expectedDistancePoint = new Vector3d(18.5, 22, 0, precisionConfiguration);
		PointToEdgeLocationKind expectedDistancePointLocationKind = PointToEdgeLocationKind.InnerPoint;
		int expectedIndex = 3;
		
		//act
		IPointToPolygonDistanceAlgorithm algorithm = new PointToPolygonDistanceAlgorithm(precisionConfiguration);
		IPointToPolygonDistanceResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertTrue(precisionConfiguration.compareWithAccuracy(expectedDistance, result.distance()) == 0, "Not expected distance");
		assertEquals(expectedDistancePoint, result.polygonDistancePoint(), "Not expected distance point");		
		assertEquals(expectedDistancePointLocationKind, result.distancePointLocationKind(), "Not expected location kind.");
		assertEquals(expectedIndex, result.edgeIndex(), "Not expected Edge Index.");
	}
}
