package cga;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import precision.PrecisionConfiguration;
import precision.model.IPrecisionConfiguration;
import spatial.algorithms.pointToPolygonLocation.PointToPolygonLocationAlgorithm;
import spatial.geometry.Polygon3d;
import spatial.geometry.Vector3d;
import spatial.model.algorithms.pointToPolygonLocation.IPointToPolygonLocationAlgorithm;
import spatial.model.algorithms.pointToPolygonLocation.IPointToPolygonLocationResult;
import spatial.model.algorithms.pointToPolygonLocation.PointToPolygonLocationKind;

class PointToPolygonLocationFixture
{

	private static IPrecisionConfiguration precisionConfiguration;
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception
	{
		precisionConfiguration = new PrecisionConfiguration(4, 0.0001);
	}

	@Test
	void testCalculate01()
	{
		//inner edge node is the nearest point for testPoint
		
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(21.8908229, 18.8678233, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(22.5, 17.5, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToPolygonLocationKind expectedLocationKind = PointToPolygonLocationKind.Inside;
		
		//act
		IPointToPolygonLocationAlgorithm algorithm = new PointToPolygonLocationAlgorithm(precisionConfiguration);
		IPointToPolygonLocationResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}

	@Test
	void testCalculate02()
	{
		//edge end node is the nearest point for testPoint
		
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(21.8908229, 18.8678233, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(22.0, 15.5, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToPolygonLocationKind expectedLocationKind = PointToPolygonLocationKind.Inside;
		
		//act
		IPointToPolygonLocationAlgorithm algorithm = new PointToPolygonLocationAlgorithm(precisionConfiguration);
		IPointToPolygonLocationResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate03()
	{
		//2 edge inner nodes are the nearest point for testPoint
		
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(21.8908229, 18.8678233, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(22.2151402, 22.0340975, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToPolygonLocationKind expectedLocationKind = PointToPolygonLocationKind.Outside;
		
		//act
		IPointToPolygonLocationAlgorithm algorithm = new PointToPolygonLocationAlgorithm(precisionConfiguration);
		IPointToPolygonLocationResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate04()
	{
		//edge end node is the nearest point for testPoint
		
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(21.8908229, 18.8678233, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(43.3261275, 30.3276997, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToPolygonLocationKind expectedLocationKind = PointToPolygonLocationKind.Outside;
		
		//act
		IPointToPolygonLocationAlgorithm algorithm = new PointToPolygonLocationAlgorithm(precisionConfiguration);
		IPointToPolygonLocationResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
	
	@Test
	void testCalculate05()
	{
		//on-boundary
		
		//arrange
		Polygon3d polygon = new Polygon3d(new Vector3d[] {
				new Vector3d(10, 10, 0, precisionConfiguration),
				new Vector3d(30, 10, 0, precisionConfiguration),
				new Vector3d(37.5, 25, 0, precisionConfiguration),
				new Vector3d(21.8908229, 18.8678233, 0, precisionConfiguration),
				new Vector3d(12.5, 25, 0, precisionConfiguration)
		});
		Vector3d testPoint = new Vector3d(33.7500, 17.5000, 0, precisionConfiguration);
		Vector3d planeNormalVector = new Vector3d(0, 0, 1, precisionConfiguration);
		
		PointToPolygonLocationKind expectedLocationKind = PointToPolygonLocationKind.OnBoundary;
		
		//act
		IPointToPolygonLocationAlgorithm algorithm = new PointToPolygonLocationAlgorithm(precisionConfiguration);
		IPointToPolygonLocationResult result = algorithm.calculate(planeNormalVector, polygon, testPoint);
		
		//assert
		assertFalse(result.hasErrors(), "Algorithm resulted with errors.");
		
		assertEquals(expectedLocationKind, result.locationKind(), "Not expected point location kind.");
	}
}
