package application;
	
import javafx.application.Application;
import javafx.geometry.Point3D;
import javafx.stage.Stage;
import javafx.scene.Group;
import javafx.scene.PerspectiveCamera;
import javafx.scene.Scene;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Cylinder;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;


public class Main extends Application 
{
	// common
	final Group root = new Group();
    final Xform world = new Xform();
    
    // for camera
    final PerspectiveCamera camera = new PerspectiveCamera(true);
    final Xform cameraXform = new Xform();
    final Xform cameraXform2 = new Xform();
    final Xform cameraXform3 = new Xform();
    final double cameraDistance = 450;
    
    // axis group
    final Group axisGroup = new Group();
    
	@Override
	public void start(Stage primaryStage) 
	{
		System.out.println("start");
        buildScene();
        buildCamera();
        buildAxes();
 
        Scene scene = new Scene(root, 1024, 768, true);
        scene.setFill(Color.GREY);
 
        primaryStage.setTitle("Sample Application");
        primaryStage.setScene(scene);
        primaryStage.show();
        scene.setCamera(camera);
	}
	
	public static void main(String[] args) 
	{
		System.setProperty("prism.dirtyopts", "false");
		launch(args);
	}
	
	private void buildScene() 
	{
        System.out.println("buildScene");
        root.getChildren().add(world);
    }
	
	private void buildCamera() 
	{
		camera.setNearClip(0.1);
		camera.setFarClip(10000.0);
		camera.setTranslateX(10);
        camera.setTranslateZ(-150);
        camera.setFieldOfView(80);
		
        Xform cameraTransform = new Xform();
        cameraTransform.getChildren().add(camera);
        cameraTransform.ry.setAngle(-0);
        cameraTransform.rx.setAngle(-15.0 - 90);
        cameraTransform.rz.setAngle(-240);
        
        //root.getChildren().add(cameraXform);
        //cameraXform.getChildren().add(cameraXform2);
        //cameraXform2.getChildren().add(cameraXform3);
        //cameraXform3.getChildren().add(camera);
        //cameraXform3.setRotateZ(180.0);
 
//        camera.setNearClip(0.1);
//        camera.setFarClip(10000.0);
//        camera.setTranslateX(cameraDistance);
//        camera.setTranslateY(cameraDistance);
//        camera.setTranslateZ(cameraDistance);
//        Rotate cameraRotate = new Rotate();
//        cameraRotate.
//        camera.getTransforms().add(new Rotate());
        //camera.setTranslateX(200);
        //camera.setTranslateX(-30);
        //cameraXform.ry.setAngle(60);
        //cameraXform.rx.setAngle(180);
        //cameraXform.rz.setAngle(30);
    }
	
	private void buildAxes() 
	{
        System.out.println("buildAxes()");
        final PhongMaterial redMaterial = new PhongMaterial();
        redMaterial.setDiffuseColor(Color.DARKRED);
        redMaterial.setSpecularColor(Color.RED);
 
        final PhongMaterial greenMaterial = new PhongMaterial();
        greenMaterial.setDiffuseColor(Color.DARKGREEN);
        greenMaterial.setSpecularColor(Color.GREEN);
 
        final PhongMaterial blueMaterial = new PhongMaterial();
        blueMaterial.setDiffuseColor(Color.DARKBLUE);
        blueMaterial.setSpecularColor(Color.BLUE);
		
		Cylinder xAxis = createConnection(new Point3D(0, 0, 0), new Point3D(100, 0 ,0));
		Cylinder yAxis = createConnection(new Point3D(0, 0, 0), new Point3D(0, 100 ,0));
		Cylinder zAxis = createConnection(new Point3D(0, 0, 0), new Point3D(0, 0 , 100));
		
		xAxis.setMaterial(redMaterial);
		yAxis.setMaterial(greenMaterial);
		zAxis.setMaterial(blueMaterial);
		
		axisGroup.getChildren().addAll(xAxis, yAxis, zAxis);
		world.getChildren().addAll(axisGroup);
    }
	
	
	
	private Cylinder createConnection(Point3D origin, Point3D target) 
	{
	    Point3D yAxis = new Point3D(0, 1, 0);
	    Point3D diff = target.subtract(origin);
	    double height = diff.magnitude();

	    Point3D mid = target.midpoint(origin);
	    Translate moveToMidpoint = new Translate(mid.getX(), mid.getY(), mid.getZ());

	    Point3D axisOfRotation = diff.crossProduct(yAxis);
	    double angle = Math.acos(diff.normalize().dotProduct(yAxis));
	    Rotate rotateAroundCenter = new Rotate(-Math.toDegrees(angle), axisOfRotation);

	    Cylinder line = new Cylinder(0.2, height);

	    line.getTransforms().addAll(moveToMidpoint, rotateAroundCenter);

	    return line;
	}
}
